<?php
/**
* NOTICE OF LICENSE
*
* YOU MAY NOT REDISTRIBUTE THIS SOFTWARE IN ANY WAY.
*
* YOU MAY NOT MODIFY THIS SOFTWARE IN ANY WAY.
*
* THERE IS NO WARRANTY FOR THIS SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW.
* EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THIS SOFTWARE “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
* FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS
* SOFTWARE IS WITH YOU. SHOULD THIS SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF
* ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
*
* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY
* COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS THIS SOFTWARE AS
* PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
* INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THIS
* SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE
* OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THIS SOFTWARE TO OPERATE
* WITH ANY OTHER SOFTWARE), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGES.
*
* IF THE DISCLAIMER OF WARRANTY AND LIMITATION OF LIABILITY PROVIDED ABOVE CANNOT BE
* GIVEN LOCAL LEGAL EFFECT ACCORDING TO THEIR TERMS, REVIEWING COURTS SHALL APPLY LOCAL
* LAW THAT MOST CLOSELY APPROXIMATES AN ABSOLUTE WAIVER OF ALL CIVIL LIABILITY IN CONNECTION
* WITH THIS SOFTWARE, UNLESS A WARRANTY OR ASSUMPTION OF LIABILITY ACCOMPANIES A COPY OF THIS
* SOFTWARE IN RETURN FOR A FEE.
*
* @author    George Filippakis <george.filippakis@gmail.com>
* @copyright 2015-2017 George Filippakis
* @license   LICENSE.txt
*/

class AdminPriceditorController extends ModuleAdminController
{
    public function __construct()
    {
        $this->bootstrap = true;
        $this->useToolbar = true;
        $this->display_header = true;
        
        $this->valid_filters = array(
            'id_product',
            'id_product_attribute',
            'name',
            'reference',
            'ean',
            'upc',
            'name_category',
            'manufacturer',
            'supplier',
            'status',
        );
        $this->valid_order_by = array(
            'id_product',
            'id_product_attribute',
            'name',
            'reference',
            'ean',
            'upc',
            'name_category',
            'manufacturer',
            'supplier',
            'status',
        );
        $this->valid_order_way = array(
            'asc',
            'desc',
        );
        
        $this->_pagination = array(20, 50, 100, 300, 1000);
        
        $this->_default_pagination = 50;
        
        $this->_defaultOrderBy = 'id_product';
        
        parent::__construct();
    }
    
    public function setMedia()
    {
        parent::setMedia();
        
        $this->addJs(_MODULE_DIR_.$this->module->name.'/views/js/priceditor.js');
        $this->addCSS(_MODULE_DIR_.$this->module->name.'/views/css/priceditor.css', 'all', 9999);
    }
    
    public function init()
    {
        parent::init();

        if (version_compare('1.5.0.0', _PS_VERSION_) <= 0 &&
            version_compare('1.6.0.0', _PS_VERSION_) > 0) {
            $this->is_ps_15 = true;
        } else {
            $this->is_ps_15 = false;
        }
        
        if (version_compare('1.6.0.0', _PS_VERSION_) <= 0 &&
            version_compare('1.7.0.0', _PS_VERSION_) > 0) {
            $this->is_ps_16 = true;
        } else {
            $this->is_ps_16 = false;
        }
        
        if (version_compare('1.7.0.0', _PS_VERSION_) <= 0 &&
            version_compare('1.8.0.0', _PS_VERSION_) > 0) {
            $this->is_ps_17 = true;
        } else {
            $this->is_ps_17 = false;
        }
    }
    
    public function initContent()
    {
        $order_by = null;
        $order_way = null;
        
        if (Tools::getIsset('priceditorOrderby') && ! Tools::getIsset('submitResetpriceditor')) {
            if (in_array(Tools::getValue('priceditorOrderby'), $this->valid_order_by)) {
                $order_by = Tools::getValue('priceditorOrderby');
                $this->context->cookie->{'priceditor_Orderby'} = $order_by;
                $this->_orderBy = $order_by;
            }
        } elseif ($order_by = $this->context->cookie->{'priceditor_Orderby'}) {
            if (in_array($order_by, $this->valid_order_by)) {
                $this->_orderBy = $order_by;
            }
        }
        if (empty($order_by) && isset($this->context->cookie->{'priceditor_Orderby'})) {
            unset($this->context->cookie->{'priceditor_Orderby'});
        }
        
        if (Tools::getIsset('priceditorOrderway') && ! Tools::getIsset('submitResetpriceditor')) {
            if (in_array(Tools::getValue('priceditorOrderway'), $this->valid_order_way)) {
                $order_way = Tools::getValue('priceditorOrderway');
                $this->context->cookie->{'priceditor_Orderway'} = $order_way;
                $this->_orderWay = $order_way;
            }
        } elseif ($order_way = $this->context->cookie->{'priceditor_Orderway'}) {
            if (in_array($order_way, $this->valid_order_way)) {
                $this->_orderWay = $order_way;
            }
        }
        if (empty($order_way) && isset($this->context->cookie->{'priceditor_Orderway'})) {
            unset($this->context->cookie->{'priceditor_Orderway'});
        }
        
        foreach ($this->valid_filters as $filter) {
            if (Tools::getIsset('priceditorFilter_'.$filter)) {
                $value = Tools::getValue('priceditorFilter_'.$filter);
                if ($value === '' || Tools::getIsset('submitResetpriceditor')) {
                    unset($this->context->cookie->{'priceditorFilter_'.$filter});
                } else {
                    $this->context->cookie->{'priceditorFilter_'.$filter} = $value;
                }
            }
        }
        if (Tools::getIsset('submitFilterpriceditor')) {
            if (Tools::getValue('submitFilterpriceditor') && ! Tools::getIsset('submitResetpriceditor')) {
                $this->context->cookie->submitFilterpriceditor = Tools::getValue('submitFilterpriceditor');
            } else {
                unset($this->context->cookie->submitFilterpriceditor);
            }
        }
        
        parent::initContent();
    }
    
    public function renderList()
    {
        $context = Context::getContext();
        $db = Db::getInstance();
        if (is_object($context->link)) {
            $link = $context->link;
        } elseif (!is_object($link)) {
            $link = new Link();
        }
        $id_lang = $context->language->id;
        $address = Address::initialize();
        
        if (Shop::isFeatureActive()) {
            $is_multistore = true;
        } else {
            $is_multistore = false;
        }
        
        if ($is_multistore) {
            if (Shop::getContext() !== Shop::CONTEXT_SHOP) {
                $this->warnings[] = $this->l(
                    'You have enabled the Multistore feature and selected a shop group as your context. '.
                    'Changing values could affect more than one stores. Please proceed with caution.'
                );
            }
        }
        
        $is_ps_15 = $this->is_ps_15;
        $is_ps_16 = $this->is_ps_16;
        $is_ps_17 = $this->is_ps_17;
        
        $display_settings = array(
            'images' => false || Configuration::get('PRICEDITOR_DISPLAY_IMAGES'),
            'names' => false || Configuration::get('PRICEDITOR_DISPLAY_NAMES'),
            'references' => false || Configuration::get('PRICEDITOR_DISPLAY_REFERENCES'),
            'categories' => false || Configuration::get('PRICEDITOR_DISPLAY_CATEGORIES'),
            'attributes' => false || Configuration::get('PRICEDITOR_DISPLAY_ATTRIBUTES'),
            'ean' => false || Configuration::get('PRICEDITOR_DISPLAY_EAN'),
            'upc' => false || Configuration::get('PRICEDITOR_DISPLAY_UPC'),
            'manufacturer' => false || Configuration::get('PRICEDITOR_DISPLAY_MANUFACTURER'),
            'supplier' => false || Configuration::get('PRICEDITOR_DISPLAY_SUPPLIER'),
            'status' => false || Configuration::get('PRICEDITOR_DISPLAY_STATUS'),
            'image_type' => Configuration::get('PRICEDITOR_IMAGE_TYPE'),
            'use_retail_price' => false || Configuration::get('PRICEDITOR_USE_RETAIL_PRICE'),
        );
        
        $imageTypeObj = false;
        if (is_numeric($display_settings['image_type']) && $display_settings['image_type'] > 0) {
            $imageTypeObj = new ImageType((int)$display_settings['image_type']);
        }
        if (!Validate::isLoadedObject($imageTypeObj)) {
            $display_settings['images'] = false;
        }
        
        $order_by = null;
        $order_way = null;
        
        $sql_filters = array();
        $filters = $this->context->cookie->getFamily('priceditorFilter_');
        foreach ($filters as $key => $value) {
            $newKey = str_replace('priceditorFilter_', '', $key);
            $filters[$newKey] = $value;
            switch ($key) {
                case 'priceditorFilter_id_product':
                    if (! empty($value)) {
                        $sql_filters[] = ' p.`id_product` = '.(int)$value;
                    }
                    break;
                case 'priceditorFilter_id_product_attribute':
                    if (! empty($value)) {
                        $sql_filters[] = ' pa.`id_product_attribute` = '.(int)$value;
                    }
                    break;
                case 'priceditorFilter_name':
                    if (! empty($value)) {
                        $sql_filters[] = ' pl.`name` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\'';
                    }
                    break;
                case 'priceditorFilter_reference':
                    if (! empty($value)) {
                        $sql_filters[] = ' (pa.`reference` <> \'\' AND pa.`reference` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\')
                            OR (p.`reference` <> \'\' AND p.`reference` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\')';
                    }
                    break;
                case 'priceditorFilter_ean':
                    if (! empty($value)) {
                        $sql_filters[] = ' (pa.`ean13` <> \'\' AND pa.`ean13` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\')
                            OR (p.`ean13` <> \'\' AND p.`ean13` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\')';
                    }
                    break;
                case 'priceditorFilter_upc':
                    if (! empty($value)) {
                        $sql_filters[] = ' (pa.`upc` <> \'\' AND pa.`upc` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\')
                            OR (p.`upc` <> \'\' AND p.`upc` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\')';
                    }
                    break;
                case 'priceditorFilter_name_category':
                    if (! empty($value)) {
                        $sql_filters[] = ' cl.`name` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\'';
                    }
                    break;
                case 'priceditorFilter_manufacturer':
                    if (! empty($value)) {
                        $sql_filters[] = ' m.`name` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\'';
                    }
                    break;
                case 'priceditorFilter_supplier':
                    if (! empty($value)) {
                        $sql_filters[] = ' s.`name` LIKE \'%'.
                            str_replace(array('%', '_', ' '), array('\\%', '\\_', '%'), $value).
                            '%\'';
                    }
                    break;
                case 'priceditorFilter_status':
                        $sql_filters[] = ' product_shop.`active` = '.(int)$value;
                    break;
            }
        }
        $filters_has_value = false;
        foreach ($this->valid_filters as $filter) {
            if (! isset($filters[$filter])) {
                $filters[$filter] = '';
            } else {
                $filters_has_value = true;
            }
        }
        
        if (Tools::getValue('reset_filter_category')) {
            $this->context->cookie->id_category_priceditor_filter = false;
        }
        if (Shop::isFeatureActive() && $this->context->cookie->id_category_priceditor_filter) {
            $category = new Category((int)$this->context->cookie->id_category_priceditor_filter);
            if (!$category->inShop()) {
                $this->context->cookie->id_category_priceditor_filter = false;
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminPriceditor'));
            }
        }
        
        $this->id_current_category = 0;
        if ($id_category = (int)Tools::getValue('id_category')) {
            $this->id_current_category = $id_category;
            if ($is_ps_16) {
                $this->context->cookie->id_category_priceditor_filter = $id_category;
            }
            if ($is_ps_17) {
                $this->context->cookie->id_category_priceditor_filter = $id_category;
            }
        } elseif ($id_category = $this->context->cookie->id_category_priceditor_filter) {
            $this->id_current_category = $id_category;
        }

        if ($id_category = (int)$this->id_current_category) {
            self::$currentIndex .= '&id_category='.(int)$this->id_current_category;
        }

        if (!$id_category) {
            $id_category = Configuration::get('PS_ROOT_CATEGORY');
        }
        
        $base_url = preg_replace('#&id_category=[0-9]*#', '', self::$currentIndex).'&token='.$this->token;
        $is_category_filter = (bool)$this->id_current_category;
        if ($is_ps_15) {
            $helper = new Helper();
            $category_tree = $helper->renderCategoryTree(
                null, // root
                array((int)$id_category), // selected cat
                'categoryBox', // input name
                true, // use radio
                false, // use search
                array(), // disabled categories
                false, // use in popup
                true // use shop context
            );
        } elseif ($is_ps_16) {
            $tree = new HelperTreeCategories('categories-tree', $this->l('Filter by category'));
            $tree->setHeaderTemplate('../../controllers/products/helpers/tree/tree_header.tpl')
                ->setTemplate('../../controllers/products/helpers/tree/tree_categories.tpl')
                ->setAttribute('is_category_filter', $is_category_filter)
                ->setAttribute('base_url', $base_url)
                ->setInputName('id-category')
                ->setSelectedCategories(array((int)$id_category));
            $category_tree = $tree->render();
        } elseif ($is_ps_17) {
            $tree = new HelperTreeCategories('categories-tree', $this->l('Filter by category'));
            $tree->setAttribute('is_category_filter', $is_category_filter)
                ->setAttribute('base_url', $base_url)
                ->setInputName('id-category')
                ->setRootCategory(Category::getRootCategory()->id)
                ->setSelectedCategories(array((int)$id_category));
            $category_tree = $category_tree = $tree->render();
        }

        if (isset($this->context->cookie->priceditor_pagination) &&
            $this->context->cookie->priceditor_pagination) {
            $limit = $this->context->cookie->priceditor_pagination;
        } else {
            $limit = $this->_default_pagination;
        }
        
        if (empty($order_by)) {
            if ($this->context->cookie->{'priceditor_Orderby'}) {
                $order_by = $this->context->cookie->{'priceditor_Orderby'};
            } elseif ($this->_orderBy) {
                $order_by = $this->_orderBy;
            } else {
                $order_by = $this->_defaultOrderBy;
            }
        }

        if (empty($order_way)) {
            if ($this->context->cookie->{'priceditor_Orderway'}) {
                $order_way = $this->context->cookie->{'priceditor_Orderway'};
            } elseif ($this->_orderWay) {
                $order_way = $this->_orderWay;
            } else {
                $order_way = $this->_defaultOrderWay;
            }
            $order_way = Tools::strtoupper($order_way);
        }
        
        $limit = (int)Tools::getValue('priceditor_pagination', $limit);
        if (in_array($limit, $this->_pagination) && $limit != $this->_default_pagination) {
            $this->context->cookie->priceditor_pagination = $limit;
        } else {
            unset($this->context->cookie->priceditor_pagination);
        }
            
        $start = 0;
        if ((int)Tools::getValue('priceditor_page')) {
            $start = ((int)Tools::getValue('priceditor_page') - 1) * $limit;
        }
        
        if ((int)Tools::getValue('submitFilterpriceditor')) {
            $start = ((int)Tools::getValue('submitFilterpriceditor') - 1) * $limit;
        }

        if ($start) {
            $this->context->cookie->{'priceditor_start'} = $start;
        } elseif (isset($this->context->cookie->{'priceditor_start'})) {
            unset($this->context->cookie->{'priceditor_start'});
        }
        
        $query = new DbQuery();
        
        $query->select('SQL_CALC_FOUND_ROWS p.`id_product` AS `id_product`');
        $query->select('IFNULL(pa.`id_product_attribute`, \'\') AS `id_product_attribute`');
        $query->select('pl.`name` AS `name`');
        $query->select('cl.`name` AS `name_category`');
        $query->select('IF(pa.`reference` <> \'\', pa.`reference`, p.`reference`) AS `reference`');
        $query->select('IF(pa.`ean13` <> \'\', pa.`ean13`, p.`ean13`) AS `ean13`');
        $query->select('IF(pa.`upc` <> \'\', pa.`upc`, p.`upc`) AS `upc`');
        $query->select('m.`name` AS `manufacturer`');
        $query->select('s.`name` AS `supplier`');
        $query->select('product_shop.`active` AS `active`');
        $query->select('product_shop.`price` AS `product_price`');
        $query->select(
            'IF(product_attribute_shop.`id_product_attribute`, 
            product_attribute_shop.`price`, 
            0) AS `combination_price`'
        );
        $query->select('product_shop.`id_tax_rules_group`');
        
        $query->from(Product::$definition['table'], 'p');
        
        $query->join(Shop::addSqlAssociation(
            Product::$definition['table'],
            'p',
            true
        ));
        
        $on = 'p.`id_product` = pl.`id_product` AND pl.`id_lang` = '.(int)$id_lang;
        $on .= Shop::addSqlRestrictionOnLang('pl');
        $query->leftJoin(Product::$definition['table'].'_lang', 'pl', '('.$on.')');
        
        if ($this->id_current_category) {
            $on = 'cp.`id_product` = p.`id_product` AND cp.`id_category` = '.(int)$this->id_current_category;
            $query->innerJoin('category_product', 'cp', '('.$on.')');
        }
        
        $on = 'p.`id_product` = pa.`id_product`';
        $query->leftJoin(Combination::$definition['table'], 'pa', '('.$on.')');
        
        $query->join(Shop::addSqlAssociation(
            Combination::$definition['table'],
            'pa',
            false
        ));
        
        $on = 'p.`id_category_default` = cl.`id_category` AND cl.`id_lang` = '.(int)$id_lang;
        $on .= Shop::addSqlRestrictionOnLang('cl');
        $query->leftJoin(Category::$definition['table'].'_lang', 'cl', '('.$on.')');
        
        $on = 'p.`id_manufacturer` = m.`id_manufacturer`';
        $query->leftJoin(Manufacturer::$definition['table'], 'm', '('.$on.')');
        
        $on = 'p.`id_supplier` = s.`id_supplier`';
        $query->leftJoin(Supplier::$definition['table'], 's', '('.$on.')');
        
        if (count($sql_filters)) {
            foreach ($sql_filters as $sql_filter) {
                $query->where($sql_filter);
            }
        }
        
        switch ($order_by) {
            case 'id_product':
                $query->orderBy('p.`id_product` '.$order_way);
                break;
            case 'id_product_attribute':
                $query->orderBy('pa.`id_product_attribute` '.$order_way);
                break;
            case 'name':
                $query->orderBy('pl.`name` '.$order_way);
                break;
            case 'reference':
                $query->orderBy('p.`reference` '.$order_way);
                $query->orderBy('pa.`reference` '.$order_way);
                break;
            case 'ean':
                $query->orderBy('p.`ean13` '.$order_way);
                $query->orderBy('pa.`ean13` '.$order_way);
                break;
            case 'upc':
                $query->orderBy('p.`upc` '.$order_way);
                $query->orderBy('pa.`upc` '.$order_way);
                break;
            case 'name_category':
                $query->orderBy('cl.`name` '.$order_way);
                break;
            case 'manufacturer':
                $query->orderBy('m.`name` '.$order_way);
                break;
            case 'supplier':
                $query->orderBy('s.`name` '.$order_way);
                break;
            case 'status':
                $query->orderBy('product_shop.`active` '.$order_way);
                break;
            default:
                $query->orderBy('p.`id_product` '.$order_way);
        }
        
        $query->groupBy(Product::$definition['table'].'_shop.`id_product`');
        $query->groupBy(Combination::$definition['table'].'_shop.`id_product_attribute`');
        
        if (is_numeric($start) && (int)$start >= 0 &&
            is_numeric($limit) && (int)$limit > 0) {
            $query->limit((int)$limit, (int)$start);
        }
        
        $products = $db->executeS($query->build());
        if (! is_array($products)) {
            $products = array();
        }
        
        $this->_listTotal = $db->getValue('SELECT FOUND_ROWS()', false);
                
        $full_products = array();
        $price_fields = array(
            'product_price' => 6,
            'retail_price' => defined('_PS_PRICE_COMPUTE_PRECISION_') ?
                _PS_PRICE_COMPUTE_PRECISION_ : 2,
            'combination_price' => 6,
            'combination_retail_price' => defined('_PS_PRICE_COMPUTE_PRECISION_') ?
                _PS_PRICE_COMPUTE_PRECISION_ : 2,
        );
        foreach ($products as $row) {
            $p_data = array();
            
            $p_data['id_product'] = $row['id_product'];
            
            if (! empty($row['id_product_attribute'])) {
                $isCombination = true;
                $p_data['id_product_attribute'] = $row['id_product_attribute'];
            } else {
                $isCombination = false;
                $p_data['id_product_attribute'] = '0';
            }
            $p_data['reference'] = $row['reference'];
            $p_data['ean13'] = $row['ean13'];
            $p_data['upc'] = $row['upc'];
            $p_data['manufacturer'] = $row['manufacturer'];
            $p_data['supplier'] = $row['supplier'];
            $p_data['status'] = $row['active'];
                
            $p_data['name'] = $row['name'];
            
            $p_data['category'] = $row['name_category'];
            
            if ($display_settings['images']) {
                $query = new DbQuery();
                $query->select('i.`id_image`');
                $query->from('image', 'i');
                if ($is_ps_15) {
                    $query->join(Shop::addSqlAssociation('image', 'i', true));
                    $query->where('i.`cover` = 1');
                } else {
                    $query->join(Shop::addSqlAssociation('image', 'i', true, 'image_shop.cover = 1'));
                }
                $query->where('i.`id_product` = '.(int)$row['id_product']);
                $cover_image = $db->getValue($query->build());
                if ($cover_image) {
                    $row['product_image'] = (int)$cover_image;
                } else {
                    $row['product_image'] = '';
                }
            }
            
            $p_data['attributes'] = array();
            if ($isCombination) {
                if ($display_settings['images']) {
                    $query = new DbQuery();
                    $query->select('`id_image`');
                    $query->from('product_attribute_image', 'pai');
                    $query->where('pai.`id_product_attribute` = '.(int)$row['id_product_attribute']);
                    $query->orderBy('`id_image` DESC');
                    $row['combination_image'] = $db->getValue($query->build());
                }
                
                if ($display_settings['attributes']) {
                    $query = new DbQuery();
                    
                    $query->select('agl.`name` AS group_name');
                    $query->select('al.`name` AS attribute_name');
                    
                    $query->from(Combination::$definition['table'], 'pa');
                    
                    $query->join(Shop::addSqlAssociation(
                        Combination::$definition['table'],
                        'pa',
                        true
                    ));
                    
                    $on = 'pac.`id_product_attribute` = pa.`id_product_attribute`';
                    $query->leftJoin('product_attribute_combination', 'pac', '('.$on.')');
                    
                    $on = 'a.`id_attribute` = pac.`id_attribute`';
                    $query->leftJoin('attribute', 'a', '('.$on.')');
                    
                    $on = 'ag.`id_attribute_group` = a.`id_attribute_group`';
                    $query->leftJoin('attribute_group', 'ag', '('.$on.')');
                    
                    $on = 'a.`id_attribute` = al.`id_attribute` AND al.`id_lang` = '.(int)$id_lang;
                    $query->leftJoin('attribute_lang', 'al', '('.$on.')');
                    
                    $on = 'ag.`id_attribute_group` = agl.`id_attribute_group` AND agl.`id_lang` = '.(int)$id_lang;
                    $query->leftJoin('attribute_group_lang', 'agl', '('.$on.')');
                    
                    $query->where('pa.`id_product` = '.(int)$row['id_product']);
                    $query->where('pa.`id_product_attribute` = '.(int)$row['id_product_attribute']);
                    
                    $query->groupBy('ag.`id_attribute_group`');
                    
                    $attributes = $db->executeS($query->build());
                    if (! is_array($attributes)) {
                        $attributes = array();
                    }
                    
                    foreach ($attributes as $data) {
                        $p_data['attributes'][] = $data['group_name'].': '.
                            $data['attribute_name'];
                    }
                }
            }

            $tax_manager = TaxManagerFactory::getManager($address, $row['id_tax_rules_group']);
            $tax_calculator = $tax_manager->getTaxCalculator();

            $tax_rate = $tax_calculator->getTotalRate();
            if ($tax_rate < 0) {
                $tax_rate = 0;
            }
            $p_data['tax_rate'] = Tools::ps_round($tax_rate, 2);
            $p_data['tax_rate_raw'] = $tax_rate;
            
            if ($isCombination) {
                $p_data['combination_price'] = Tools::ps_round(
                    $row['combination_price'],
                    6
                );
                $p_data['combination_retail_price'] = Tools::ps_round(
                    (1 + 0.01 * $tax_rate) * $row['combination_price'],
                    defined('_PS_PRICE_COMPUTE_PRECISION_') ? _PS_PRICE_COMPUTE_PRECISION_ : 2
                );
            } else {
                $p_data['combination_price'] = 0;
                $p_data['combination_retail_price'] = 0;
            }
            
            $p_data['product_price'] = Tools::ps_round(
                $row['product_price'],
                6
            );
            $p_data['retail_price'] = Tools::ps_round(
                (1 + 0.01 * $tax_rate) * $row['product_price'],
                defined('_PS_PRICE_COMPUTE_PRECISION_') ? _PS_PRICE_COMPUTE_PRECISION_ : 2
            );
            
            foreach ($price_fields as $field => $precision) {
                $p_data[$field] = sprintf("%.".$precision."f", $p_data[$field]);
            }
            
            if ($display_settings['images']) {
                if ($isCombination && $row['combination_image']) {
                    $p_data['image_link'] = $link->getImageLink(
                        '-',
                        $row['id_product'].'-'.$row['combination_image'],
                        $display_settings['image_type'] ? $imageTypeObj->name : false
                    );
                } else {
                    $p_data['image_link'] = $link->getImageLink(
                        '-',
                        $row['id_product'].'-'.$row['product_image'],
                        $display_settings['image_type'] ? $imageTypeObj->name : false
                    );
                }
            }
            
            $full_products[] = $p_data;
        }
                
        $page = (int)Tools::getValue('priceditor_page', 1);
        if ($page == 0) {
            $page = 1;
        }

        $pagination = $this->_default_pagination;
        if (in_array((int)Tools::getValue('priceditor_pagination'), $this->_pagination)) {
            $pagination = (int)Tools::getValue('priceditor_pagination');
        } elseif (isset($this->context->cookie->priceditor_pagination) &&
            $this->context->cookie->priceditor_pagination) {
            $pagination = $this->context->cookie->priceditor_pagination;
        }

        $total_pages = max(1, ceil($this->_listTotal / $pagination));

        if ($page > $total_pages) {
            $page = $total_pages;
        }

        $selected_pagination = Tools::getValue(
            'priceditor_pagination',
            (
                isset($this->context->cookie->priceditor_pagination) ?
                $this->context->cookie->priceditor_pagination : $this->_default_pagination
            )
        );
        
        if ($this->is_ps_15) {
            $tpl = $this->context->smarty->createTemplate($this->getTemplatePath().'product-list-15.tpl');
        }
        if ($this->is_ps_16) {
            $tpl = $this->context->smarty->createTemplate($this->getTemplatePath().'product-list-16.tpl');
        }
        if ($this->is_ps_17) {
            $tpl = $this->context->smarty->createTemplate($this->getTemplatePath().'product-list-17.tpl');
        }
        
        if (! is_array($this->toolbar_title) ||
            count($this->toolbar_title) == 0) {
            $this->initBreadcrumbs();
            $this->initToolbarTitle();
        }
        if (is_array($this->toolbar_title) &&
            count($this->toolbar_title)) {
            if (!$is_ps_15) {
                $this->toolbar_title = implode(
                    ' '.Configuration::get('PS_NAVIGATION_PIPE').' ',
                    $this->toolbar_title
                );
                
                $this->toolbar_title = str_replace(
                    $this->module->displayName,
                    '',
                    $this->toolbar_title
                );
            }
        }
        
        $colspan = 5;
        foreach ($display_settings as $col => $display) {
            if ($col == 'image_type') {
                continue;
            }
            if ($display) {
                $colspan += 1;
            }
        }
        $display_settings['colspan'] = $colspan;
        
        $tpl->assign(array(
            'products' => $full_products,
            'token' => $this->token,
            'action' => self::$currentIndex.'&token='.$this->token,
            'ajax_post_url' => rawurlencode($this->context->link->getAdminLink('AdminPriceditor')),
            'page' => $page,
            'total_pages' => $total_pages,
            'selected_pagination' => $selected_pagination,
            'pagination' => $this->_pagination,
            'list_total' => $this->_listTotal,
            'list_id' => 'priceditor',
            'table' => 'priceditor',
            'is_ps_15' => $is_ps_15,
            'is_ps_16' => $is_ps_16,
            'is_ps_17' => $is_ps_17,
            'show_toolbar' => true,
            'toolbar_btn' => array(),
            'toolbar_scroll' => false,
            'title' => $this->toolbar_title,
            'currentIndex' => self::$currentIndex,
            'baseIndex' => __PS_BASE_URI__.basename(_PS_ADMIN_DIR_).'/index.php',
            'is_category_filter' => $is_category_filter,
            'category_tree' => $category_tree,
            'filters' => $filters,
            'filters_has_value' => $filters_has_value,
            'order_by' => $order_by,
            'order_way' => $order_way,
            'row_hover' => true,
            'base_url' => rawurlencode($base_url),
            'is_multistore' => $is_multistore,
            'display_settings' => $display_settings,
        ));
        
        return $tpl->fetch();
    }

    public function addFiltersToBreadcrumbs()
    {
        $filtered_by = array();
        $filters = $this->context->cookie->getFamily('priceditorFilter_');
        foreach ($filters as $key => $value) {
            switch ($key) {
                case 'priceditorFilter_id_product':
                    $filter_key = $this->l('Product ID');
                    break;
                case 'priceditorFilter_id_product_attribute':
                    $filter_key = $this->l('Combination ID');
                    break;
                case 'priceditorFilter_name':
                    $filter_key = $this->l('Name');
                    break;
                case 'priceditorFilter_reference':
                    $filter_key = $this->l('Reference');
                    break;
                case 'priceditorFilter_ean':
                    $filter_key = $this->l('EAN13');
                    break;
                case 'priceditorFilter_upc':
                    $filter_key = $this->l('UPC');
                    break;
                case 'priceditorFilter_name_category':
                    $filter_key = $this->l('Category');
                    break;
                case 'priceditorFilter_manufacturer':
                    $filter_key = $this->l('Manufacturer');
                    break;
                case 'priceditorFilter_supplier':
                    $filter_key = $this->l('Supplier');
                    break;
                case 'priceditorFilter_status':
                    $filter_key = $this->l('Status');
                    break;
            }
            $filter_value = htmlspecialchars($value, ENT_QUOTES, 'UTF-8');
            if (! empty($filter_value)) {
                $filtered_by[] = sprintf($this->l('%s: %s'), $filter_key, $filter_value);
            }
        }

        if (count($filtered_by)) {
            return sprintf($this->l('filter by %s'), implode(', ', $filtered_by));
        }
        
        return '';
    }
    
    private function canSave()
    {
        if (!$this->checkToken()) {
            $this->errors[] = $this->l('Invalid security token');
            return false;
        }
        
        $context = Context::getContext();
        $cookie = $context->cookie;
        
        if (! Validate::isLoadedObject($context->shop)) {
            $this->errors[] = $this->l('Invalid session cookie');
            return false;
        }
        
        if (! is_object($cookie)) {
            $this->errors[] = $this->l('Invalid session cookie');
            return false;
        }
        
        if (empty($cookie->id_employee)) {
            $this->errors[] = $this->l('Invalid session cookie');
            return false;
        }
        
        $employee = new Employee($cookie->id_employee);
        
        if (! Validate::isLoadedObject($employee)) {
            $this->errors[] = $this->l('Invalid session cookie');
            return false;
        }
        
        if (empty($cookie->passwd)) {
            $this->errors[] = $this->l('Invalid session cookie');
            return false;
        }

        if ($employee->passwd != $cookie->passwd) {
            $this->errors[] = $this->l('Invalid session cookie');
            return false;
        }
        
        if (Shop::getContext() == Shop::CONTEXT_SHOP) {
            if (! $employee->hasAuthOnShop($context->shop->id)) {
                $this->errors[] = $this->l('Permission denied');
                return false;
            }
        } elseif (! Validate::isLoadedObject($context->shop)) {
            $this->errors[] = $this->l('Permission denied');
            return false;
        }
        
        $products_tab_id = Tab::getIdFromClassName('AdminProducts');
        $access = Profile::getProfileAccess($employee->id_profile, $products_tab_id);
        if (isset($access['edit']) && $access['edit']) {
            return true;
        }
        
        $this->errors[] = $this->l('Permission denied');
        return false;
    }
    
    public function ajaxProcessSavePrice()
    {
        $this->json = true;
        $this->product_price_changed = false;
        $this->id_product_changed = 0;
        $this->combination_price_changed = false;
        $this->id_combination_changed = 0;
        
        if (! $this->canSave()) {
            $this->status = false;
            $this->errors[] = $this->l('The price could not be updated');
            return;
        }
        
        $id_product = Tools::isSubmit('id_product') ?
            (int)Tools::getValue('id_product', 0) : 0;
            
        if (! is_numeric($id_product) || ! ($id_product > 0)) {
            $this->status = false;
            $this->errors[] = $this->l('Invalid product data');
            return;
        }
        
        $id_product_attribute = Tools::isSubmit('id_product_attribute') ?
            (int)Tools::getValue('id_product_attribute', 0) : 0;
            
        if (! is_numeric($id_product_attribute) || ! ($id_product_attribute >= 0)) {
            $this->status = false;
            $this->errors[] = $this->l('Invalid product data');
            return;
        }
        
        $use_retail_price = Configuration::get('PRICEDITOR_USE_RETAIL_PRICE');
        
        $product_price = Tools::isSubmit('product_price') ?
            Tools::getValue('product_price', 0) : 0;
        
        $product_retail_price = Tools::isSubmit('product_retail_price') ?
            Tools::getValue('product_retail_price', 0) : 0;
            
        $combination_price = Tools::isSubmit('combination_price') ?
            Tools::getValue('combination_price', 0) : 0;
            
        $combination_retail_price = Tools::isSubmit('combination_retail_price') ?
            Tools::getValue('combination_retail_price', 0) : 0;
            
        if (! Validate::isPrice($product_price)) {
            $this->status = false;
            $this->errors[] = $this->l('Invalid product price');
            return;
        }
            
        if (! Validate::isPrice($product_retail_price)) {
            $this->status = false;
            $this->errors[] = $this->l('Invalid product price');
            return;
        }
        
        if (! Validate::isNegativePrice($combination_price)) {
            $this->status = false;
            $this->errors[] = $this->l('Invalid combination price');
            return;
        }
        
        if (! Validate::isNegativePrice($combination_retail_price)) {
            $this->status = false;
            $this->errors[] = $this->l('Invalid combination price');
            return;
        }

        $address = Address::initialize();
        
        if ($id_product_attribute > 0) {
            try {
                $combination = new Combination($id_product_attribute);
                if ($combination->id_product != $id_product) {
                    $this->status = false;
                    $this->errors[] = $this->l('Invalid product data');
                    return;
                } elseif ($id_product == 0) {
                    $this->status = false;
                    $this->errors[] = $this->l('Invalid product data');
                    return;
                }
                
                $product = new Product($combination->id_product);
                
                $tax_manager = TaxManagerFactory::getManager($address, $product->id_tax_rules_group);
                $tax_calculator = $tax_manager->getTaxCalculator();

                $tax_rate = $tax_calculator->getTotalRate();
                if ($tax_rate < 0) {
                    $tax_rate = 0;
                }
                
                $combination_initial_retail_price = $combination->price * (1 + 0.01 * $tax_rate);
                
                if ($use_retail_price) {
                    $combination->price = sprintf(
                        "%.6f",
                        ($combination_retail_price / (1 + 0.01 * $tax_rate))
                    );
                } else {
                    $combination->price = $combination_price;
                }
                
                $combination_new_retail_price = $combination->price * (1 + 0.01 * $tax_rate);
                
                if ($combination->update()) {
                    $this->status = true;
                    $this->combination_price_changed = true;
                    $this->id_combination_changed = $combination->id;
                    $this->id_product_changed = $combination->id_product;
                } else {
                    $this->status = false;
                    $this->errors[] = $this->l('Error while saving combination price');
                }
                
                $product_initial_retail_price = $product->price * (1 + 0.01 * $tax_rate);
                
                if ($use_retail_price) {
                    $product->price = sprintf(
                        "%.6f",
                        ($product_retail_price / (1 + 0.01 * $tax_rate))
                    );
                } else {
                    $product->price = $product_price;
                }
                
                $product_new_retail_price = $product->price * (1 + 0.01 * $tax_rate);
                
                if ($product->update()) {
                    $this->status = true;
                    $this->product_price_changed = true;
                    $this->id_product_changed = $product->id;
                    $this->product = $product;
                } else {
                    $this->status = false;
                    $this->errors[] = $this->l('Error while saving product price');
                }
            } catch (Exception $e) {
                $this->status = false;
                $this->errors[] = $e->getMessage();
            }
        } elseif ($id_product > 0) {
            try {
                $product = new Product($id_product);
                
                $tax_manager = TaxManagerFactory::getManager($address, $product->id_tax_rules_group);
                $tax_calculator = $tax_manager->getTaxCalculator();

                $tax_rate = $tax_calculator->getTotalRate();
                if ($tax_rate < 0) {
                    $tax_rate = 0;
                }
                
                $product_initial_retail_price = $product->price * (1 + 0.01 * $tax_rate);
                
                if ($use_retail_price) {
                    $product->price = sprintf(
                        "%.6f",
                        ($product_retail_price / (1 + 0.01 * $tax_rate))
                    );
                } else {
                    $product->price = $product_price;
                }
                
                $product_new_retail_price = $product->price * (1 + 0.01 * $tax_rate);
                
                if ($product->update()) {
                    $this->status = true;
                    $this->product_price_changed = true;
                    $this->id_product_changed = $product->id;
                    $this->product = $product;
                } else {
                    $this->status = false;
                    $this->errors[] = $this->l('Error while saving product price');
                }
            } catch (Exception $e) {
                $this->status = false;
                $this->errors[] = $e->getMessage();
            }
        }
        
        if (property_exists($this, 'combination_price_changed') && $this->combination_price_changed) {
            if ($this->is_ps_15) {
                if (Validate::isLoadedObject($this->context->employee)) {
                    $employee = $this->context->employee->firstname;
                    $employee .= ' '.$this->context->employee->lastname;
                    $employee .= ' ('.$this->context->employee->email.') ';
                } else {
                    $employee = '';
                }
                $message = sprintf(
                    $this->l('changed combination retail impact from %s to %s'),
                    sprintf("%.2f", $combination_initial_retail_price),
                    sprintf("%.2f", $combination_new_retail_price)
                );
                Logger::addLog(
                    $employee.$message,
                    1,
                    null,
                    'Combination',
                    $id_product_attribute,
                    true
                );
            }
            if ($this->is_ps_16) {
                PrestaShopLogger::addLog(
                    sprintf(
                        $this->l('Combination retail impact changed from %s to %s'),
                        sprintf("%.2f", $combination_initial_retail_price),
                        sprintf("%.2f", $combination_new_retail_price)
                    ),
                    1,
                    null,
                    'Combination',
                    $id_product_attribute,
                    true
                );
            }
            if ($this->is_ps_17) {
                PrestaShopLogger::addLog(
                    sprintf(
                        $this->l('Combination retail impact changed from %s to %s'),
                        sprintf("%.2f", $combination_initial_retail_price),
                        sprintf("%.2f", $combination_new_retail_price)
                    ),
                    1,
                    null,
                    'Combination',
                    $id_product_attribute,
                    true
                );
            }
        }
        
        if (property_exists($this, 'product_price_changed') && $this->product_price_changed) {
            if ($this->is_ps_15) {
                if (Validate::isLoadedObject($this->context->employee)) {
                    $employee = $this->context->employee->firstname;
                    $employee .= ' '.$this->context->employee->lastname;
                    $employee .= ' ('.$this->context->employee->email.') ';
                } else {
                    $employee = '';
                }
                $message = sprintf(
                    $this->l('changed product retail price from %s to %s'),
                    sprintf("%.2f", $product_initial_retail_price),
                    sprintf("%.2f", $product_new_retail_price)
                );
                Logger::addLog(
                    $employee.$message,
                    1,
                    null,
                    'Product',
                    $id_product,
                    true
                );
            }
            if ($this->is_ps_16) {
                PrestaShopLogger::addLog(
                    sprintf(
                        $this->l('Product retail price changed from %s to %s'),
                        sprintf("%.2f", $product_initial_retail_price),
                        sprintf("%.2f", $product_new_retail_price)
                    ),
                    1,
                    null,
                    'Product',
                    $id_product,
                    true
                );
            }
            if ($this->is_ps_17) {
                PrestaShopLogger::addLog(
                    sprintf(
                        $this->l('Product retail price changed from %s to %s'),
                        sprintf("%.2f", $product_initial_retail_price),
                        sprintf("%.2f", $product_new_retail_price)
                    ),
                    1,
                    null,
                    'Product',
                    $id_product,
                    true
                );
            }
            return;
        }
        
        $this->status = false;
        $this->errors[] = 'Invalid product data';
    }
    
    public function displayAjaxSavePrice()
    {
        $product_price_changed = property_exists($this, 'product_price_changed') ?
            $this->product_price_changed : false;
            
        $combination_price_changed = property_exists($this, 'combination_price_changed') ?
            $this->combination_price_changed : false;
            
        $product_price = 0;
        $product_retail_price = 0;
        $combination_price = 0;
        $combination_retail_price = 0;
        
        $address = Address::initialize();
            
        if ($combination_price_changed && $this->id_product_changed && $this->id_combination_changed) {
            $tax_manager = TaxManagerFactory::getManager($address, $this->product->id_tax_rules_group);
            $tax_calculator = $tax_manager->getTaxCalculator();

            $tax_rate = $tax_calculator->getTotalRate();
            if ($tax_rate < 0) {
                $tax_rate = 0;
            }
            
            $product_price = Product::getPriceStatic(
                $this->id_product_changed, // id_product
                false, // use tax
                false, // id_product_attribute
                6, // decimals
                null, // divisor
                false, // only reduction
                false // use reduction
            );
            $product_retail_price = Product::getPriceStatic(
                $this->id_product_changed, // id_product
                true, // use tax
                false, // id_product_attribute
                defined('_PS_PRICE_COMPUTE_PRECISION_') ? _PS_PRICE_COMPUTE_PRECISION_ : 2, // decimals
                null, // divisor
                false, // only reduction
                false // use reduction
            );
            $combination_price = Combination::getPrice(
                $this->id_combination_changed // id_product_attribute
            );
            $combination_retail_price = $combination_price * (1 + 0.01 * $tax_rate);
        } elseif ($product_price_changed && $this->id_product_changed) {
            $product_price = Product::getPriceStatic(
                $this->id_product_changed, // id_product
                false, // use tax
                false, // id_product_attribute
                6, // decimals
                null, // divisor
                false, // only reduction
                false // use reduction
            );
            $product_retail_price = Product::getPriceStatic(
                $this->id_product_changed, // id_product
                true, // use tax
                false, // id_product_attribute
                defined('_PS_PRICE_COMPUTE_PRECISION_') ? _PS_PRICE_COMPUTE_PRECISION_ : 2, // decimals
                null, // divisor
                false, // only reduction
                false // use reduction
            );
        }
        
        $data = array(
            'status' => $this->status,
            'product_price_changed' => $product_price_changed,
            'combination_price_changed' => $combination_price_changed,
            'product_price' => $product_price,
            'product_retail_price' => $product_retail_price,
            'combination_impact' => $combination_price,
            'combination_retail_impact' => $combination_retail_price,
        );
        if (count($this->errors)) {
            $data['errors'] = $this->errors;
        }
        
        echo Tools::jsonEncode($data);
    }
}
