<?php
/**
* NOTICE OF LICENSE
*
* YOU MAY NOT REDISTRIBUTE THIS SOFTWARE IN ANY WAY.
*
* YOU MAY NOT MODIFY THIS SOFTWARE IN ANY WAY.
*
* THERE IS NO WARRANTY FOR THIS SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW.
* EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THIS SOFTWARE “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
* FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS
* SOFTWARE IS WITH YOU. SHOULD THIS SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF
* ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
*
* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY
* COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS THIS SOFTWARE AS
* PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
* INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THIS
* SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE
* OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THIS SOFTWARE TO OPERATE
* WITH ANY OTHER SOFTWARE), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGES.
*
* IF THE DISCLAIMER OF WARRANTY AND LIMITATION OF LIABILITY PROVIDED ABOVE CANNOT BE
* GIVEN LOCAL LEGAL EFFECT ACCORDING TO THEIR TERMS, REVIEWING COURTS SHALL APPLY LOCAL
* LAW THAT MOST CLOSELY APPROXIMATES AN ABSOLUTE WAIVER OF ALL CIVIL LIABILITY IN CONNECTION
* WITH THIS SOFTWARE, UNLESS A WARRANTY OR ASSUMPTION OF LIABILITY ACCOMPANIES A COPY OF THIS
* SOFTWARE IN RETURN FOR A FEE.
*
* @author    George Filippakis <george.filippakis@gmail.com>
* @copyright 2015-2017 George Filippakis
* @license   LICENSE.txt
*/

if (!defined('_PS_VERSION_')) {
    exit;
}
  
class PricEditor extends Module
{
    private static $configuration_values = array(
        'boolean_values' => array(
            'display_images' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_IMAGES',
                'error' => 'Invalid display image status',
                'default' => '0',
            ),
            'display_names' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_NAMES',
                'error' => 'Invalid display name status',
                'default' => '1',
            ),
            'display_references' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_REFERENCES',
                'error' => 'Invalid display reference status',
                'default' => '1',
            ),
            'display_categories' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_CATEGORIES',
                'error' => 'Invalid display category status',
                'default' => '1',
            ),
            'display_attributes' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_ATTRIBUTES',
                'error' => 'Invalid display attributes status',
                'default' => '1',
            ),
            'display_ean' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_EAN',
                'error' => 'Invalid display EAN13 status',
                'default' => '0',
            ),
            'display_upc' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_UPC',
                'error' => 'Invalid display UPC status',
                'default' => '0',
            ),
            'display_manufacturer' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_MANUFACTURER',
                'error' => 'Invalid display manufacturer status',
                'default' => '0',
            ),
            'display_supplier' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_SUPPLIER',
                'error' => 'Invalid display supplier status',
                'default' => '0',
            ),
            'display_status' => array(
                'configuration_value' => 'PRICEDITOR_DISPLAY_STATUS',
                'error' => 'Invalid display product status',
                'default' => '0',
            ),
            'use_retail_price' => array(
                'configuration_value' => 'PRICEDITOR_USE_RETAIL_PRICE',
                'error' => 'Invalid use retail price status',
                'default' => '0',
            ),
        ),
    );
    
    public function __construct()
    {
        $this->name = 'priceditor';
        $this->tab = 'quick_bulk_update';
        $this->version = '3.0.0';
        $this->author = 'George Filippakis';
        $this->need_instance = 0;
        if (version_compare('1.5.0.0', _PS_VERSION_) <= 0 &&
            version_compare('1.6.0.0', _PS_VERSION_) > 0) {
            $this->ps_versions_compliancy = array('min' => '1.5.0.0', 'max' => '1.6.0.0');
        } elseif (version_compare('1.6.0.0', _PS_VERSION_) <= 0) {
            $this->ps_versions_compliancy = array('min' => '1.5', 'max' => _PS_VERSION_);
        } else {
            $this->ps_versions_compliancy = array('min' => '', 'max' => '');
        }
        $this->bootstrap = true;
        $this->module_key = '1a618e0c45152ef54b8fb5443e8c7b21';

        parent::__construct();

        $this->displayName = $this->l('Price Editor');
        $this->description = $this->l('Edit your product and combination prices through a list of all your products.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }
    
    public function install()
    {
        if (parent::install()) {
            if ($this->registerHook('actionAdminControllerSetMedia')) {
                if ($this->installTab()) {
                    $configuration_values = array(
                        'PRICEDITOR_DISPLAY_IMAGES' => 0,
                        'PRICEDITOR_DISPLAY_NAMES' =>1,
                        'PRICEDITOR_DISPLAY_REFERENCES' => 1,
                        'PRICEDITOR_DISPLAY_CATEGORIES' => 1,
                        'PRICEDITOR_DISPLAY_ATTRIBUTES' => 1,
                        'PRICEDITOR_DISPLAY_EAN' => 0,
                        'PRICEDITOR_DISPLAY_UPC' => 0,
                        'PRICEDITOR_DISPLAY_MANUFACTURER' => 0,
                        'PRICEDITOR_DISPLAY_SUPPLIER' => 0,
                        'PRICEDITOR_DISPLAY_STATUS' => 0,
                        'PRICEDITOR_USE_RETAIL_PRICE' => 0,
                    );
                    foreach ($configuration_values as $k => $v) {
                        Configuration::updateValue($k, $v);
                    }
                    return true;
                } else {
                    $this->unregisterHook('actionAdminControllerSetMedia');
                    parent::uninstall();
                }
            } else {
                parent::uninstall();
            }
        }

        return false;
    }

    public function uninstall()
    {
        $this->unregisterHook('actionAdminControllerSetMedia');
        
        $this->uninstallTab();
        
        parent::uninstall();

        return true;
    }

    public function installTab()
    {
        $tab = new Tab();
        $tab->active = 1;
        $tab->class_name = 'AdminPriceditor';
        $tab->name = array();

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = 'Price Editor';
        }

        $tab->id_parent = Tab::getIdFromClassName('AdminCatalog');
        $tab->module = $this->name;
        return $tab->add();
    }

    public function uninstallTab()
    {
        $id_tab = (int)Tab::getIdFromClassName('AdminPriceditor');

        if ($id_tab) {
            $tab = new Tab($id_tab);
            return $tab->delete();
        } else {
            return false;
        }
    }

    public function hookActionAdminControllerSetMedia()
    {
        if (!$this->active) {
            return;
        }

        if (method_exists($this->context->controller, 'addJquery')) {
            $this->context->controller->addJquery();
        }

        $this->context->controller->addJS($this->_path.'js/priceditor.js');
        $this->context->controller->addCSS($this->_path.'css/priceditor.css');
    }
    
    public function getContent()
    {
        $output = null;
        
        if (Tools::isSubmit('submit'.$this->name)) {
            foreach (self::$configuration_values['boolean_values'] as $k => $v) {
                $$k = (string)Tools::getValue($v['configuration_value'], $v['default']);
                if (! Validate::isBool($$k)) {
                    $output .= $this->displayError($this->l($v['error']));
                } else {
                    if ($$k) {
                        Configuration::updateValue(
                            $v['configuration_value'],
                            '1'
                        );
                    } else {
                        Configuration::updateValue(
                            $v['configuration_value'],
                            '0'
                        );
                    }
                }
            }
            
            $image_type = (string)Tools::getValue('PRICEDITOR_IMAGE_TYPE', '');
            if (empty($image_type) || ! Validate::isUnsignedId($image_type)) {
                $output .= $this->displayError($this->l('Invalid image type'));
            } else {
                Configuration::updateValue(
                    'PRICEDITOR_IMAGE_TYPE',
                    (int)$image_type
                );
            }
            
            if (is_null($output)) {
                $output .= $this->displayConfirmation($this->l('Settings updated successfully'));
            }
        }
        
        return $output.$this->displayForm();
    }
    
    public function displayForm()
    {
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        
        if (version_compare('1.5.0.0', _PS_VERSION_) <= 0 &&
            version_compare('1.6.0.0', _PS_VERSION_) > 0) {
            $switch_type = 'radio';
        } else {
            $switch_type = 'switch';
        }
         
        $helper = new HelperForm();

        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->show_cancel_button = true;
        $helper->toolbar_scroll = false;
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            ),
        );

        foreach (self::$configuration_values['boolean_values'] as $k => $v) {
            $$k = Configuration::get($v['configuration_value']);
            if ($$k !== false && Validate::isBool($$k)) {
                $helper->fields_value[$v['configuration_value']] = $$k;
            } else {
                $helper->fields_value[$v['configuration_value']] = 0;
            }
        }
        
        $image_type = Configuration::get('PRICEDITOR_IMAGE_TYPE');
        if ($image_type !== false && Validate::isUnsignedId($image_type)) {
            $helper->fields_value['PRICEDITOR_IMAGE_TYPE'] = $image_type;
        } else {
            $helper->fields_value['PRICEDITOR_IMAGE_TYPE'] = '';
        }
            
        $fields_form = array();
        
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Display Settings'),
            ),
            'input' => array(
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display product images'),
                    'name' => 'PRICEDITOR_DISPLAY_IMAGES',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_images_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_images_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Image type'),
                    'name' => 'PRICEDITOR_IMAGE_TYPE',
                    'options' => array(
                        'query' => ImageType::getImagesTypes('products'),
                        'id' => 'id_image_type',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display product names'),
                    'name' => 'PRICEDITOR_DISPLAY_NAMES',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_names_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_names_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display product references'),
                    'name' => 'PRICEDITOR_DISPLAY_REFERENCES',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_references_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_references_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display product categories'),
                    'name' => 'PRICEDITOR_DISPLAY_CATEGORIES',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_categories_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_categories_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display product attributes'),
                    'name' => 'PRICEDITOR_DISPLAY_ATTRIBUTES',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_attributes_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_attributes_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display EAN13 barcodes'),
                    'name' => 'PRICEDITOR_DISPLAY_EAN',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_ean_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_ean_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display UPC barcodes'),
                    'name' => 'PRICEDITOR_DISPLAY_UPC',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_upc_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_upc_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display manufacturers'),
                    'name' => 'PRICEDITOR_DISPLAY_MANUFACTURER',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_manufacturer_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_manufacturer_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display suppliers'),
                    'name' => 'PRICEDITOR_DISPLAY_SUPPLIER',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_supplier_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_supplier_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Display product status'),
                    'name' => 'PRICEDITOR_DISPLAY_STATUS',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_status_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'display_status_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
                array(
                    'type' => $switch_type,
                    'label' => $this->l('Use retail price'),
                    'name' => 'PRICEDITOR_USE_RETAIL_PRICE',
                    'class' => $switch_type == 'radio' ? 'radioCheck' : '',
                    'desc' => $this->l(
                        'Select Yes to use prices including tax.'
                    ),
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'use_retail_price_enable',
                            'value' => 1,
                            'label' => $this->l('Enable'),
                        ),
                        array(
                            'id' => 'use_retail_price_disable',
                            'value' => 0,
                            'label' => $this->l('Disable'),
                        ),
                    ),
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'button btn btn-default pull-right'
            )
        );
         
        return $helper->generateForm($fields_form);
    }
}
