/**
* NOTICE OF LICENSE
*
* YOU MAY NOT REDISTRIBUTE THIS SOFTWARE IN ANY WAY.
*
* YOU MAY NOT MODIFY THIS SOFTWARE IN ANY WAY.
*
* THERE IS NO WARRANTY FOR THIS SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. 
* EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES 
* PROVIDE THIS SOFTWARE “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, 
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
* FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS 
* SOFTWARE IS WITH YOU. SHOULD THIS SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF 
* ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
*
* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY 
* COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS THIS SOFTWARE AS 
* PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, 
* INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THIS 
* SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE 
* OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THIS SOFTWARE TO OPERATE 
* WITH ANY OTHER SOFTWARE), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGES.
* 
* IF THE DISCLAIMER OF WARRANTY AND LIMITATION OF LIABILITY PROVIDED ABOVE CANNOT BE 
* GIVEN LOCAL LEGAL EFFECT ACCORDING TO THEIR TERMS, REVIEWING COURTS SHALL APPLY LOCAL 
* LAW THAT MOST CLOSELY APPROXIMATES AN ABSOLUTE WAIVER OF ALL CIVIL LIABILITY IN CONNECTION 
* WITH THIS SOFTWARE, UNLESS A WARRANTY OR ASSUMPTION OF LIABILITY ACCOMPANIES A COPY OF THIS 
* SOFTWARE IN RETURN FOR A FEE.
*
* @author    George Filippakis <george.filippakis@gmail.com>
* @copyright 2015-2017 George Filippakis
* @license   LICENSE.txt
*/

var ajax_post_url = 'index.php?controller=AdminPriceditorController',
priceditor_translations = {
	'invalid_price': "Invalid price",
	'error_saving_price': "Error while saving product price",
	'saved_successfully': "New price saved successfully",
	'saving': "Saving...",
	'save': "Save"
};

jQuery(document).ready(function() {
	var $ = jQuery;
	
	$('input.price-input', $('#priceditor-table')).on('keypress', function(e) {
		var save_button_id = $(this)
			.attr('id')
			.replace('-base-price', '-save')
			.replace('-retail-price', '-save')
			.replace('combination-', 'product-'),
		changed_field;
		if (e.which == 13) {
			e.preventDefault();
			$('#'+save_button_id).click();
		}
	});
	
	$('input.price-input', $('#priceditor-table')).on('keyup', function(e) {
		var input_id = $(this).attr('id').split('-'),
		id_product = input_id.length ? input_id[1] : 0,
		id_product_attribute = input_id.length ? input_id[2] : 0,
		$tax_rate = id_product ? $('#product-' + id_product + '-tax-rate') : [],
		tax_rate = $tax_rate.length ? $tax_rate.data('tax-rate') : 0,
		is_base_price = $(this).hasClass('base-price'),
		is_retail_price = $(this).hasClass('retail-price'),
		is_impact = $(this).hasClass('combination-impact'),
		is_retail_impact = $(this).hasClass('combination-retail-impact'),
		base_price,
		retail_price,
		impact,
		retail_impact,
		val = $(this).val();
		
		if (is_base_price && !isNaN(val)) {
			retail_price = parseFloat(val) * (1 + 0.01 * parseFloat(tax_rate));
			$('#product-' + id_product + '-' + id_product_attribute + '-retail-price').val(
				parseFloat(retail_price).toFixed(2)
			);
		}
		
		if (is_retail_price && !isNaN(val)) {
			base_price = parseFloat(val) / (1 + 0.01 * parseFloat(tax_rate));
			$('#product-' + id_product + '-' + id_product_attribute + '-base-price').val(
				parseFloat(base_price).toFixed(6)
			);
		}
		
		if (is_impact && !isNaN(val)) {
			retail_impact = parseFloat(val) * (1 + 0.01 * parseFloat(tax_rate));
			$('#combination-' + id_product + '-' + id_product_attribute + '-retail-price').val(
				parseFloat(retail_impact).toFixed(2)
			);
		}
		
		if (is_retail_impact && !isNaN(val)) {
			impact = parseFloat(val) / (1 + 0.01 * parseFloat(tax_rate));
			$('#combination-' + id_product + '-' + id_product_attribute + '-base-price').val(
				parseFloat(impact).toFixed(6)
			);
		}
	});
	
	$('.priceditor-save-button').on('click', function(e) {
		var id_product = $(this).data('product-id'),
		id_product_attribute = $(this).data('product-attribute-id'),
		$product_price = $('#product-'+id_product+'-'+id_product_attribute+'-base-price'),
		$product_retail_price = $('#product-'+id_product+'-'+id_product_attribute+'-retail-price'),
		product_price = $product_price.val(),
		product_retail_price = $product_retail_price.length ? $product_retail_price.val() : 0,
		$combination_price = $('#combination-'+id_product+'-'+id_product_attribute+'-base-price'),
		$combination_retail_price = $('#combination-'+id_product+'-'+id_product_attribute+'-retail-price'),
		combination_price = $combination_price.val(),
		combination_retail_price = $combination_retail_price.length ? $combination_retail_price.val() : 0,
		$icon = $('i', $(this)),
		$label = $('span.lbl', $(this)),
		$button = $(this),
		d = {};
		
		if (! $icon.hasClass('icon-save')) return;
		
		if (isNaN(product_price) || ! product_price.length) {
			showErrorMessage(priceditor_translations.invalid_price);
			$product_price.css({
				'background-color': '#ffdddd'
			});
			setTimeout(function() {
				$product_price.css({
					'background-color': 'initial'
				});
			}, 5000);
			return;
		}
		
		if ($product_retail_price.length) {
			if (isNaN(product_retail_price) || ! product_retail_price.length) {
				showErrorMessage(priceditor_translations.invalid_price);
				$product_retail_price.css({
					'background-color': '#ffdddd'
				});
				setTimeout(function() {
					$product_retail_price.css({
						'background-color': 'initial'
					});
				}, 5000);
				return;
			}
		}
		
		if (isNaN(combination_price) || ! combination_price.length) {
			showErrorMessage(priceditor_translations.invalid_price);
			$combination_price.css({
				'background-color': '#ffdddd'
			});
			setTimeout(function() {
				$combination_price.css({
					'background-color': 'initial'
				});
			}, 5000);
			return;
		}
		
		if ($combination_retail_price.length) {
			if (isNaN(combination_retail_price) || ! combination_retail_price.length) {
				showErrorMessage(priceditor_translations.invalid_price);
				$combination_retail_price.css({
					'background-color': '#ffdddd'
				});
				setTimeout(function() {
					$combination_retail_price.css({
						'background-color': 'initial'
					});
				}, 5000);
				return;
			}
		}
		
		d = {
			'ajax': 1,
			'action': 'SavePrice',
			'id_product': String(id_product),
			'id_product_attribute': String(id_product_attribute),
			'product_price': String(product_price),
			'product_retail_price': String(product_retail_price),
			'combination_price': String(combination_price),
			'combination_retail_price': String(combination_retail_price)
		}
		
		$label.html(priceditor_translations.saving).css({
			color: '#aaa'
		});;
		
		$icon
			.removeClass('icon-save')
			.addClass('icon-refresh icon-spin');
		$button.css({cursor: 'wait'});
		
		$.ajax({
			context: document.body,
			data: d,
			dataType: 'json',
			error: function(x, s, e) {
				
				$label.html(priceditor_translations.save).css({
					color: 'inherit'
				});;
				
				$icon
					.removeClass('icon-refresh icon-spin')
					.addClass('icon-save');
				$button.css({cursor: 'pointer'});
				
				showErrorMessage(e);
				return;
			},
			method: 'POST',
			success: function(d, s, x) {
				var errors, i;
				
				$label.html(priceditor_translations.save).css({
					color: 'inherit'
				});;
				
				$icon
					.removeClass('icon-refresh icon-spin')
					.addClass('icon-save');
				$button.css({cursor: 'pointer'});
				
				if (s != 'success') {
					showErrorMessage(priceditor_translations.error_saving_price);
					return;
				}
				if (d.status == 1) {
					showSuccessMessage(priceditor_translations.saved_successfully);
					if (d.product_price_changed) {
						$('input[id^="product-'+id_product+'-"].base-price', $('#priceditor-table')).val(parseFloat(d.product_price).toFixed(6));
						$('input[id^="product-'+id_product+'-"].retail-price', $('#priceditor-table')).val(parseFloat(d.product_retail_price).toFixed(2));
					}
					if (d.combination_price_changed) {
						$('input[id="combination-'+id_product+'-'+id_product_attribute+'-base-price"].combination-impact', $('#priceditor-table')).val(parseFloat(d.combination_impact).toFixed(6));
						$('input[id="combination-'+id_product+'-'+id_product_attribute+'-retail-price"].combination-retail-impact', $('#priceditor-table')).val(parseFloat(d.combination_retail_impact).toFixed(2));
					}
					return;
				}
				if (d.errors.length) {
					errors = d.errors.length;
					for (i = 0; i < errors; i++)
						showErrorMessage(d.errors[i]);
				}
			},
			timeout: 30000,
			url: ajax_post_url
		});
	});
});